﻿using System.Net;

namespace SddpViewer;

public class CidrHelper
{
  public static (IPAddress Start, IPAddress End) GetIpRangeFromCidr(string cidr)
  {
    string[] parts = cidr.Split('/');
    if (parts.Length != 2)
      throw new FormatException("Ungültiges CIDR-Format.");

    IPAddress ip = IPAddress.Parse(parts[0]);
    int prefixLength = int.Parse(parts[1]);

    if (prefixLength < 0 || prefixLength > 32)
      throw new ArgumentException("Ungültige Präfixlänge.");

    byte[] ipBytes = ip.GetAddressBytes();
    byte[] maskBytes = new byte[4];
    int remainingBits = prefixLength;

    for (int i = 0; i < maskBytes.Length; i++)
    {
      if (remainingBits >= 8)
      {
        maskBytes[i] = 0xFF;
        remainingBits -= 8;
      }
      else if (remainingBits > 0)
      {
        maskBytes[i] = (byte)(0xFF << (8 - remainingBits));
        remainingBits = 0;
      }
    }

    byte[] startBytes = new byte[4];
    byte[] endBytes = new byte[4];
    for (int i = 0; i < 4; i++)
    {
      startBytes[i] = (byte)(ipBytes[i] & maskBytes[i]);
      endBytes[i] = (byte)(ipBytes[i] | ~maskBytes[i]);
    }

    return (new IPAddress(startBytes), new IPAddress(endBytes));
  }
}
